lust = require 'lust'

local Game = Object:extend()

local reverse_acc_x = 500
local acc_x = 1500
local vel_x_max = 400
local x_drag = 300
local level_width = 400
local jump = -500
local stop_jump = -300
local min_y_diff = 50
local max_y_diff = 100
local min_width = 60
local max_width = 140
local jump_boost = -50
local min_boost_speed = 350
local floors_per_level = 50

function Game:new()
  local highscore = love.filesystem.read("highscore")

  if highscore then
    self.highscore = tonumber(highscore)
  end


  G.crt.scroll = 0
  self.circles = {}
  self.scrollboost = 0
  self.tutorial = 0
  self.vx = 0
  self.vy = 0
  self.ax = 0
  self.ay = 700
  self.grounded = false
  self.precharge_jump = 0
  self.scroll_speed = 0
  self.highest_level = 0
  self.last_gen_platform = 0
  self.score = 0
  self.combo = 0
  self.bonus_display_time = 0
  self.combotime = 0
  self.next_level_floor = floors_per_level
  self.min_width = min_width
  self.max_width = max_width

  local w, h = love.graphics.getDimensions()
  self.last_y_plat = h-30

  self.x = w/2
  self.y = h-32
  self.oldx = w/2
  self.oldy = h-32

  self.platforms = {
    { level = 0, line = {(w-level_width)/2, h-30, (w+level_width)/2, h-30 }}
  }

  for i = 1, 20 do
    self:gen_platform()
  end
end


function Game:gen_platform()
  local ydiff = love.math.random(min_y_diff, max_y_diff)
  local width = level_width
  local x = (love.graphics.getWidth() - level_width)/2
  local level = self.last_gen_platform + 1
  if level % floors_per_level ~= 0 then
    local levelscale = 0.95 ^ (math.floor(level / floors_per_level))
    width = love.math.random(self.min_width * levelscale, self.max_width * levelscale)
    x = love.math.random(0, level_width - width) + (love.graphics.getWidth() - level_width) / 2
  end

  local y = self.last_y_plat - ydiff
  self.last_y_plat = y

  table.insert(self.platforms, {
    level = level,
    line = {
      x, y, x+width, y
    }
  })
  self.last_gen_platform = level

end


function Game:draw()
end


local function clamp(x, a, b)
  if x < a then
    return a
  elseif x > b then
    return b
  else
    return x
  end
end


function Game:update(dt)
  local w, h = love.graphics.getDimensions()

  if not self.gameover then
    if self.grounded and self.y < 100 and self.scrollboost < G.crt.scroll then
      self.scrollboost = G.crt.scroll + 200 - self.y
    end
    local scrollb = 1
    if self.scrollboost > G.crt.scroll then
      scrollb = 5
    end
    G.crt.scroll = G.crt.scroll + self.scroll_speed*dt*scrollb

    if G.crt.scroll > 100 then
      self.oldy = self.oldy + 100
      self.y = self.y + 100
      self.scrollboost = self.scrollboost -100
      for i, v in ipairs(self.platforms) do
        v.line[2] = v.line[2] + 100
        v.line[4] = v.line[4] + 100
      end
      for i, c in ipairs(self.circles) do
        c.y = c.y + 100
      end
      G.crt.scroll = G.crt.scroll - 100
      self.last_y_plat = self.last_y_plat + 100
    end

    for i = #self.platforms, 1, -1 do
      local v = self.platforms[i]
      if v.line[2] > h + 30 then
        self.platforms[i] = self.platforms[#self.platforms]
        self.platforms[#self.platforms] = nil
        self:gen_platform()
      end
    end

    self.oldx = self.x
    self.oldy = self.y
    
    if self.precharge_jump > 0 then
      self.precharge_jump = self.precharge_jump - dt
    end

    if self.ax ~= 0 then
      self.vx = self.vx + dt * self.ax

      if lust.tools.sig(self.vx) ~= lust.tools.sig(self.ax) then
        self.vx = self.vx + dt * lust.tools.sig(self.ax) * reverse_acc_x
      end
    else
      local x_sig = lust.tools.sig(self.vx)
      self.vx = self.vx - x_sig * x_drag * dt

      if lust.tools.sig(self.vx) ~= x_sig then
        self.vx = 0
      end
    end

    if self.grounded then
      if self.x < self.grounded.line[1] or self.x > self.grounded.line[3] then
        self.grounded = false
      end
    end

    if not self.grounded then
      self.vy = self.vy + dt * self.ay
    end

    self.vx = clamp(self.vx, -vel_x_max, vel_x_max)
    self.x = self.x + dt * self.vx
    self.y = self.y + dt * self.vy

    local left_coll_x, left_coll_y = lust.tools.ray_line_intersection(
      (w - level_width) / 2, -100000, 0, 1,
      self.oldx, self.oldy, self.x, self.y
    )

    if left_coll_x then
      self.x = left_coll_x + 0.001
      self.y = left_coll_y
      self.vx = -0.8 * self.vx
      G.sounds.blip:play()
    else
      local right_coll_x, right_coll_y = lust.tools.ray_line_intersection(
        (w + level_width) / 2, -100000, 0, 1,
        self.oldx, self.oldy, self.x, self.y
      )
      if right_coll_x then
        self.x = right_coll_x - 0.001
        self.y = right_coll_y
        self.vx = -0.8*self.vx
        G.sounds.blip:play()
      end

    end

    if self.vy < 0 then
      for i, v in ipairs(self.platforms) do
        if v.level % floors_per_level == 0 and self.highest_level < v.level then
          local vl = v.line
          local ix, iy = lust.tools.line_line_intersection(
            vl[1], vl[2]-1, vl[3], vl[4]-1,
            self.oldx, self.oldy, self.x, self.y
          )
          if ix then
            local w20 = level_width / 20
            G.sounds.levelup:play()
            for i = 1, 20 do
              table.insert(self.circles, {x = (w-level_width)/2 + w20 *i , y = v.line[2], r = -2*i, re = 30, grow = 100})
            end
          end
        end
      end
    end

    if self.vy > 0 then
      for i, v in ipairs(self.platforms) do
        local vl = v.line
        local ix, iy = lust.tools.line_line_intersection(
          vl[1], vl[2]-1, vl[3], vl[4]-1,
          self.oldx, self.oldy, self.x, self.y
        )
        if ix then
          G.sounds.land:play()
          if self.highest_level >= v.level - 1 then
            if self.combo > 0 then
              local bonus = (self.combo - 1) * (self.score - self.combo_start_score)
              G.sounds.getbonus:play()
              self.score = self.score + bonus
              self.bonus_display_time = 1
              self.last_bonus = bonus
              self.combo = 0
            end
          end
          if self.highest_level < v.level then
            if self.score == 0 then
              self.scroll_speed = 50
            else
              if v.level >= self.next_level_floor then
                self.next_level_floor = self.next_level_floor + floors_per_level
                self.scroll_speed = self.scroll_speed * 1.1
              end
            end
            local skip = (v.level - self.highest_level)
            if skip > 0 then
              self.score = self.score + skip * 10
            end
            self.highest_level = v.level
            if skip > 1 then
              G.sounds.bonus:stop()
              self.combo = self.combo + 1
              self.combotime = 1
              if self.combo == 1 then
                self.combo_start_score = self.score
              end

              if self.combo > 1 then
                G.sounds.bonus:setPitch((2^(1.0/12.0))^(self.combo - 1))
                G.sounds.bonus:play()
              end
            end
          end
          self.vy = 0
          self.x = ix
          self.y = iy
          self.grounded = v

          if self.precharge_jump > 0 then
            self.precharge_jump = 0
            self:jump()
          end
        end
      end
    end

    if self.grounded and self.combotime > 0 then
      self.combotime = self.combotime - dt
      if self.combotime <= 0 then
        local bonus = (self.combo - 1) * (self.score - self.combo_start_score)
        self.score = self.score + bonus
        if bonus > 0 then
          G.sounds.getbonus2:play()
        end
        self.bonus_display_time = 1
        self.last_bonus = bonus
        self.combo = 0
      end

      if self.combo > 1 then
        G.crt:line(0.1, {w / 2 - level_width / 2 * math.max(0, self.combotime-0.1), 32-G.crt.scroll, w/2 + level_width/2 * math.max(0,self.combotime-0.1), 32-G.crt.scroll})
      end
    end

    if self.y > h + 30 then
      self.gameover = true
      if not self.highscore or (self.score > self.highscore) then
        self.newhighscore = true
        love.filesystem.write("highscore", tostring(self.score))
      end
      G.sounds.die:play()
    end
    
    for i, v in ipairs(self.platforms) do
      G.crt:line(0.3, v.line)
      if v.level % floors_per_level == 0 then
        G.crt:print(G.font, 0.2, v.level, w/2, math.floor(v.line[2]+5+G.crt.scroll), level_width / 2 - 5, 'right')
      end
    end
    G.crt:line(0.1, { (w - level_width) / 2, -G.crt.scroll, (w - level_width) / 2.0, h-G.crt.scroll})
    G.crt:line(0.1, { (w + level_width) / 2, -G.crt.scroll, (w + level_width) / 2.0, h-G.crt.scroll})
    if self.vy == 0 then
      G.crt:line(20, {self.x - 2.5, self.y - 3.5, self.x + 2.5, self.y - 3.5,
                      self.x + 2.5, self.y, self.x - 2.5, self.y,
                      self.x - 2.5, self.y - 3.5})
    else
      G.crt:line(20, {self.oldx, self.oldy, self.x, self.y})
    end

    G.crt:print(G.font, 0.5, self.score, 0, 10, w, 'center')

    if self.tutorial == 0 then
      G.crt:print(G.font, 0.5, "<- MOVE ->", 0, h-24, w, 'center')
    elseif self.tutorial == 1 then
      G.crt:print(G.font, 0.5, "[X] JUMP", 0, h-24, w, 'center')
    end

    if self.tutorial < 2 then
      G.crt:print(G.titlefont, 5, "PHOSPHORUS", 0, h/4, w, 'center')
      if self.highscore then
        G.crt:print(G.font, 5, "HIGHSCORE: " .. self.highscore,0, h/4+60, w, 'center')
      end
    end

    if self.bonus_display_time > 0 and self.last_bonus > 0 and self.combo <= 1 then
      G.crt:print(G.font, 1, "BONUS: " .. self.last_bonus, 0, 35, w, 'center')
      self.bonus_display_time = self.bonus_display_time - dt
    end

    if self.combo > 1 then
      G.crt:print(G.font, 0.5, self.combo .. "x COMBO", 0, 35, w, 'center')
    end

    for i = #self.circles, 1, -1 do
      local c = self.circles[i]
      c.r = c.r + dt*c.grow
      if (lust.tools.sig(c.grow) > 0 and c.r > c.re) or (lust.tools.sig(c.grow) < 0 and c.r < c.re)  then
        self.circles[i] = self.circles[#self.circles]
        self.circles[#self.circles] = nil
      else
        if c.r > 0 then
          G.crt:circle(0.5, c.x, c.y, c.r)
        end
      end
    end

  else
    G.crt:print(G.font, 1.0, "GAME OVER", 0, h / 2 - 30, w, 'center')
    G.crt:print(G.font, 0.2, "SCORE: " .. self.score, 0, h / 2 + 10, w, 'center')
    if self.newhighscore then
      G.crt:print(G.font, 3, "NEW HIGHSCORE", 0, h / 2 + 40, w, 'center')
    end
    G.crt:print(G.font, 0.5, "[R] RESTART", 0, h-24, w, 'center')
  end

end


function Game:keypressed(key)
  if self.gameover then
    
  else
    if key == 'right' then
      self.ax = self.ax + acc_x
      if self.tutorial == 0 then
        self.tutorial = 1
      end
    elseif key == 'left' then
      self.ax = self.ax - acc_x
      if self.tutorial == 0 then
        self.tutorial = 1
      end
    elseif key == 'x' then
      if self.tutorial == 1 then
        self.tutorial = 2
      end
      if self.grounded then
        self:jump()
      else
        self.precharge_jump = 0.2
      end
    end
  end
end

function Game:keyreleased(key)
  if self.gameover then
  else
    if key == 'right' then
      self.ax = self.ax - acc_x
    elseif key == 'left' then
      self.ax = self.ax + acc_x
    elseif (not self.grounded) and key == 'x' then
      if self.vy < stop_jump then
        self.vy = stop_jump
      end
    end
  end

end


function Game:jump()
  self.vy = jump
  if math.abs(self.vx) > min_boost_speed then
    self.vy = self.vy + jump_boost
    table.insert(self.circles, {x = self.x, y = self.y, r = 15, re = 0, grow = -100})
  end

  if self.y + G.crt.scroll > love.graphics.getHeight() then
    G.sounds.save:play()
  elseif self.vy == jump then
    G.sounds.jump1:play()
  else
    G.sounds.jump2:play()
  end
  self.grounded = false
end


return Game
