--
-- lust
--
-- Copyright (c) 2014, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

--- Very minimalistic camera class
local Camera = Object:extend()

--- Create a new Camera. Call Camera(...) to call and obtain the new instance
--  @param w The width of the viewport
--  @param g The height of the viewport
--  @param cx The x coordinate of the viewport's center
--  @param cy The y coordinate of the viewport's center
function Camera:new(w,h,cx,cy)
  self.x = cx
  self.y = cy
  self.w = w
  self.h = h
  self.tween = flux.group()
  self.shakeAmount = 0.0
  self.limits = {0,0,1000000,1000000}
end

--- Set up love's transformation matrix to use this camera
function Camera:setup()
  love.graphics.origin()
  local sa = self.shakeAmount
  local r = love.math.random
  local f = math.floor
  local py = - self.y  + self.h / 2
  local ym = - self.limits[4] + self.h
  if py < ym then
    py = ym
  end

  if py > self.limits[2] then
    py = self.limits[2]
  end

  local px = - self.x  + self.w / 2
  local xm = - self.limits[3] + self.w
  if px < xm then
    px = xm
  end

  if px > self.limits[1] then
    px = self.limits[1]
  end
  self.y = -py + self.h / 2
  self.x = -px + self.w / 2
  love.graphics.translate(f(r() * sa + px),
                          f(r() * sa + py))
  love.graphics.setScissor(0,0, self.w, self.h)
end

function Camera:update(dt)
  self.tween:update(dt)
end

function Camera:shake(amount)
  self.shakeAmount = amount
end


function Camera:limit(x1, y1, x2, y2)
  self.limits = {x1, y1, x2, y2}
end

function Camera:shakeTween(amount, time)
  self.tween:to(self, time or 1, {shakeAmount = amount})
end


function Camera:shakeDecay(amount, time)
  self.shakeAmount = amount
  self.tween:to(self, time or 1, {shakeAmount = 0})
end

return Camera
