--
-- lust
--
-- Copyright (c) 2014, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

--- Simple collision callback manager
local CollisionDispatcher = Object:extend()

--- Create new CollisionDispatcher. Call CollisionDispatcher() to call this
--  function and obtain the new instance
function CollisionDispatcher:new()
  self._callbacks = {}
end

--- Internal function called by the CollisionManager when it detects a 
--  collision. The dispatche will check for the availability of a collision
--  callback that accepts either (shape_a, shape_b) or (shape_b, shape_a).
--  Shapes are required to have a field shape.kind that it used for selecting 
--  the right collision callback. The optional field shape.entity will be 
--  passed to the callback.
--  @param shape_a The hardon collider shape of the first colliding object
--  @param shape_b The hardon collider shape of the second collidin object
function CollisionDispatcher:_start_collision(shape_a, shape_b)
  local cb = self._callbacks[shape_a.kind .. "/" .. shape_b.kind]
  if cb then
    cb(shape_a.entity, shape_b.entity)
    return
  end

  cb = self._callbacks[shape_b.kind .. "/" .. shape_a.kind]
  if cb then
    cb(shape_b.entity, shape_a.entity)
  end
end

function CollisionDispatcher:_stop_collision(shape_a, shape_b)
  -- TODO write me
end

--- Add a new callback for two shape kinds. The order of the given kinds odes
--  not matter. The dispatcher will always look for both combinations.
--  @param kind_a   The kind of the first shape for which the callback is 
--                  to be registered. Must be a string.
--  @param kind_b   The kind of the second shape for which the callback is 
--                  to be registered. Must be a string.
--  @param callback The callback function to be called when a collision is
--                  detected. The callback must take two parameters, which are
--                  the entities taking part in the collision. The order of the
--                  entities will always be so that arg1.shape.kind == kind_a
--                  and arg2.shape.kinde == kind_b
function CollisionDispatcher:add_callback(kind_a, kind_b, callback)
  self._callbacks[kind_a .. "/" .. kind_b] = callback
end

return CollisionDispatcher
