Entities = Object:extend()

function Entities:new(img, maxcount)
  self.img = G.core.images:load(img)
  self.batch = love.graphics.newSpriteBatch(self.img, maxcount or 1)
  self.individuals = {}
  self.oldIndividuals = {}
  self.toRemove = {}
end


function Entities:update(dt, ...)
  local i = 1
  local updateIndividual = self.updateIndividual
  while i <= #self.individuals do
    updateIndividual(self, i, dt, ...)
    i = i + 1
  end
  
  table.sort(self.toRemove)
  -- remove in reverse order to make sure indices of elements to be removed are still valid
  for ri = #self.toRemove, 1, -1 do
    local idx = self.toRemove[ri]
    self.individuals[idx] = self.individuals[#self.individuals]
    self.oldIndividuals[idx] = self.oldIndividuals[#self.oldIndividuals]
    self.individuals[#self.individuals] = nil
    self.oldIndividuals[#self.oldIndividuals] = nil
  end
  self.toRemove = {}
end


function Entities:insert(data, dt)
  dt = dt or 0
  print(dt)
  local last = {
    posx = data.posx - dt * data.velx,
    posy = data.posy - dt * data.vely,
    rot  = data.rot,
    velx = data.velx,
    vely = data.vely,
    velr = data.velr
  }

  table.insert(self.individuals, data)
  table.insert(self.oldIndividuals, last)
end


function Entities:remove(idx)
  table.insert(self.toRemove, idx)
end


function Entities:updateIndividual(idx, dt)
  local a   = self.individuals[idx]
  local old = self.oldIndividuals[idx]
  old.posx = a.posx
  old.posy = a.posy
  old.rot  = a.rot
  old.velx = a.velx
  old.vely = a.vely

  a.rot  = a.rot  + dt * a.velr
  a.posx = a.posx + dt * a.velx
  a.posy = a.posy + dt * a.vely
end


function Entities:draw(lerp)
  self.batch:bind()
  self.batch:clear()
  while #self.individuals > self.batch:getBufferSize() do
    self.batch:setBufferSize(self.batch:getBufferSize() * 2)
  end

  for i, a in ipairs(self.individuals) do
    local old = self.oldIndividuals[i]
    self.batch:add(
      a.quad,
      lust.tools.lerp(old.posx, a.posx, lerp),
      lust.tools.lerp(old.posy, a.posy, lerp),
      lust.tools.lerp(old.rot,  a.rot,  lerp),
      1,
      1,
      a.offsetx,
      a.offsety
    )
  end
  self.batch:unbind()
  love.graphics.draw(self.batch)
end
