--
-- lust
--
-- Copyright (c) 2014, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local Entity = require('lust.entity')

local Group = Entity:extend()

function Group:new(state)
  Group.super.new(self, state)
  self.children = {}
  self.updatechildren = {}
end


function Group:update(dt)
  local count = 0
  for i, e in pairs(self.updatechildren) do
    count = count + 1
    if e.dead then
      e:die()
      self.children[e] = nil
      self.updatechildren[e] = nil
    else
      e:update(dt)
    end
  end
end

function Group:updateGfx(dt)
  for i, e in pairs(self.updatechildren) do
		e:updateGfx(dt)
  end
end



function Group:draw(lerp)
  if self.sorted then
    local sorted = self:getSortedEntities()
    for _, e in ipairs(sorted) do
      e:draw(lerp)
    end
  else
    for _, e in pairs(self.children) do
      e:draw(lerp)
    end
  end
end


function Group:getSortedEntities(sort)
  local sorted = {}
  for i, e in pairs(self.children) do
    table.insert(sorted, e)
  end
  table.sort(sorted, sort or function(a,b) return a.layer < b.layer or (a.layer == b.layer and  a.pos.y < b.pos.y) end)
  return sorted
end


function Group:add(entity)
  self.children[entity] = entity
  if not entity.noupdates then
    self.updatechildren[entity] = entity
  end
end


function Group:remove(entity)
  entity:die()
  self.children[entity] = nil
end


function Group:find(op)
  for i, e in pairs(self.children) do
    if op(e) then
      return e
    end
  end
end

function Group:remove_if(op)
  for i, e in pairs(self.children) do
    if op(e) then
      self:remove(e)
    end
  end
end


function Group:findNearest(op, x, y)
  local nearest = nil
  for i, e in pairs(self.children) do
    if op(e) then
      if not nearest then
        nearest = e
      else
        if tools.sq(nearest.pos.x-x) + tools.sq(nearest.pos.y-y) > tools.sq(e.pos.x - x) + tools.sq(e.pos.y - y) then
          nearest = e
        end
      end
    end
  end

  return nearest
end

return Group
