--
-- lust
--
-- Copyright (c) 2014, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

--- Simple input manager
local InputManager = Object:extend()

--- Create new input manager. Call InputManager(...) to call constructor and
--  obtain new instance.
function InputManager:new()
  self.key_press_map   = {}
  self.key_release_map = {}

  self.pressed = {}
end

--- Pass the key press event to a registered callback for that key, if one
--  exists.
--  @param key The key to be passed to the callback
--  @return true if a matching callback was found, false otherwise (useful for
--          chaining InputManagers)
function InputManager:dispatch_press(key)
  self.pressed[key] = key
  if self.key_press_map[key] then
    self.key_press_map[key](key)
    return true
  end
  return false
end

--- Pass the key release event to a registered callback for that key, if one
--  exists.
--  @param key The key to be passed to the callback
--  @return true if a matching callback was found, false otherwise (useful for
--          chaining InputManagers)
function InputManager:dispatch_release(key)
  -- Make sure we don't get key release events that
  -- belong to other InputManager's key press events
  if not self.pressed[key] then
    return false
  end
  self.pressed[key] = nil
  if self.key_release_map[key] then
    self.key_release_map[key](key)
    return true
  end
  return false
end

--- Add new callback for key events.
--  @param key Keycode (as defined by love) to register the events for
--  @param press_cb (optional)   The callback to call when the key is pressed
--                               If the parameter is missing or nil, it will be
--                               removed from the callbacks
--  @param release_cb (optional) The callback to call when the key is pressed
--                               If the parameter is missing or nil, it will be
--                               removed from the callbacks
function InputManager:add_key(key, press_cb, release_cb)
  self.key_press_map[key] = press_cb
  self.key_release_map[key] = release_cb
end

return InputManager
