--
-- lust
--
-- Copyright (c) 2014, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local InputManager = Object:extend()

function InputManager:new()
  self.stack = {}
end


function InputManager:prepare()
  love.keypressed = function(key) self:key_pressed(key) end
  love.keyreleased = function(key) self:key_released(key) end
  love.textinput = function(char) self:text_input(char) end
end


function InputManager:key_pressed(key)
  if #self.stack > 0 then
    self.stack[#self.stack]:dispatch_press(key)
  end
end


function InputManager:key_released(key)
  if #self.stack > 0 then
    self.stack[#self.stack]:dispatch_release(key)
  end
end


function InputManager:text_input(char)
  if #self.stack > 0 then
    self.stack[#self.stack]:text_input(char)
  end
end


function InputManager:push(context)
  table.insert(self.stack, context)
  self.stack[#self.stack]:prepare()
end


function InputManager:pop()
  table.remove(self.stack)
  if #self.stack > 0 then
    self.stack[#self.stack]:prepare()
  end
end

return InputManager
