--
-- lust
--
-- Copyright (c) 2014, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local Menu = require('lust.menu')

local KeybindingMenu = Menu.Screen:extend()

KeybindingMenu.KeyEvent = Object:extend()

function KeybindingMenu.KeyEvent:new(event, description, default)
  self.event = event
  self.description = description
  self.key = default
end


function KeybindingMenu:update()
  self.keyentries = {}
  for i, e in ipairs(self.keymap:getActions()) do
    table.insert(self.keyentries, Menu.Entry('[' .. (e[3] ~= " " and e[3] or "space").. ']'))
  end
end


function KeybindingMenu:new(keymap)
  KeybindingMenu.super.new(self, "Keyboard Configuration")
  
  self.keymap = keymap
  self.entries = {}

  for i, e in ipairs(self.keymap:getActions()) do
    table.insert(self.entries, Menu.Entry(e[2], function() self:readKey(i) end))
  end

  self:update()

  local backEntry = Menu.Entry("Back")
  backEntry.offset = 1
  table.insert(self.entries, backEntry)

end


function KeybindingMenu:readKey(i)
  self.readingKey = i
  self.lkp = love.keypressed
  self.lkr = love.keyreleased
  love.keypressed = function(key) self:assignKey(key) end
  love.keyreleased = function()end
end


function KeybindingMenu:save()
  print("Saving Keyboard Mapping")
end


function KeybindingMenu:assignKey(key)
  if key == "unknown" then
    return
  end

  love.keypressed = self.lkp
  love.keyreleased = self.lkr
  if key ~= "escape" then
  --[[
    for i,e in ipairs(self.keymap:getActions()) do
      if e[3] == key then
        e[3] = ''
      end
    end
    ]]
    --self.keymap:getActions()[self.readingKey][3] = key
    self.keymap:unmap(key)
    self.keymap:remap(key, self.keymap:getActions()[self.readingKey][1])
    self:update()
  end

  self.readingKey = nil
end


function KeybindingMenu:draw(style)
  if self.readingKey then
    style:drawCaption("Assign key for '" .. self.keymap:getActions()[self.readingKey][2] .. "'", true)
  else
    style:drawTitle("Keyboard Configuration")
    local w = KeybindingMenu.super.drawEntries(self, style, self.entries)
    style.x = w + 20
    style.y = 0
    style:drawTitle()
    KeybindingMenu.super.drawEntries(self, style, self.keyentries)
  end
  
end


function KeybindingMenu:enter(menu)
  KeybindingMenu.super.enter(self, menu)
end

return KeybindingMenu
