--
-- lust
--
-- Copyright (c) 2014, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local MenuStyle = Object:extend()


MenuStyle.State = Object:extend()

function MenuStyle.State:new(style)
  self.y = 0
  self.style = style
  self.x = style.x or 0
end

function MenuStyle:new(titlefont, titlecolor, entryfont, entrycolor, selectedcolor, width, height)
  self.titleFont = titlefont or love.graphics.getFont()
  self.titleColor = titlecolor or {255,255,255,255}
  self.entryFont = entryfont or self.titleFont
  self.entryColor = entrycolor or {255,255,255,128}
  self.selectedColor = selectedcolor or {255,255,255,255}
  self.width = width or love.graphics.getWidth()
  self.height = height or love.graphics.getHeight()
end


function MenuStyle:newState()
  return MenuStyle.State(self)
end

function MenuStyle.State:offset(offset)
  self.y = self.y + offset * self.style.entryFont:getHeight()
end

function MenuStyle.State:drawTitle(title)
  if self.style.skipTitle then
    return
  end


  if title then
    love.graphics.setColor(unpack(self.style.titleColor))
    love.graphics.setFont(self.style.titleFont)
    love.graphics.print(title, self.x, self.y)
  end

  self.y = self.y + self:getTitleHeight()
end


function MenuStyle.State:getTitleHeight()
  return self.style.titleFont:getHeight() + (self.style.titleOffset or 0)
end


function MenuStyle.State:drawCaption(title, selected)
  if selected then
    love.graphics.setColor(unpack(self.style.selectedColor))
  else
    love.graphics.setColor(unpack(self.style.entryColor))
  end

  love.graphics.setFont(self.style.entryFont)
  love.graphics.print(title, math.floor(self.x or 0), math.floor(self.y))
  self:offset(1)
  return self.style.entryFont:getWidth(title)
end

function MenuStyle.State:rowsPerScreen()
  return math.floor( (self.style.height - self:getTitleHeight()) / self.style.entryFont:getHeight()) - 2
end


return MenuStyle
