local StateMachine = Object:extend()

local State = Object:extend()

function State:new(machine, name)
  self.name = name
  self.machine = machine
  self.transitions = {}
end


function StateMachine:new()
  self.states = {}
end


function StateMachine:state(name)
  local newState = State(self, name)
  self.states[name] = newState
  return newState
end


function State:onEnter(func)
  self.onEnterFunc = func
  return self
end


function State:onLeave(func)
  self.onLeaveFunc = func
  return self
end


function State:enter(...)
  if self.onEnter then
    self.onEnter(self, ...)
  end
end


function State:leave(...)
  if self.onLeave then
    self.onLeave(self, ...)
  end
end


function State:transition(event, dstState, action, guard)
  local t = self.transitions[event]
  if not t then
    t = {}
    self.transitions[event] = t
  end
  t[#t+1] = {dstState, guard, action}
  return self
end


function State:timeout(time, dstState, action, guard)
  self:transition("frame", dstState, action,
    function(cs, ...)
      return time <= cs.machine.currentTime and (not guard or guard(cs, ...))
    end)

  return self
end


function State:state(name)
  return self.machine:state(name)
end


function StateMachine:frame(dt)
  self.currentTime = self.currentTime + dt
end


function StateMachine:postEvent(event, ...)
  if event == "frame" then
    self:frame(...)
  end

  local cs = self.currentState
  local evts = cs.transitions[event]
  if evts then
    for _, evt in ipairs(evts) do
      if not evt[2] or evt[2](cs, ...) then
        if evt[1] ~= self.name then
          cs:leave()
        end

        evt[3](cs, ...)
        
        self.currentState = self.states[evt[1]]
        assert(self.currentState)

        if evt[1] ~= self.name then
          self.currentState:enter(...)
        end

        self.currentTime = 0

        break
      end
    end
  end
end


function StateMachine:setState(state, ...)
  self.currentState = self.states[state]

  assert(self.currentState)

  self.currentState:enter(...)
end


function State:postEvent(event, ...)
  self.machine:postEvent(event, ...)
end

return StateMachine
