--
-- lust
--
-- Copyright (c) 2014, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local StateManager = Object:extend()


function StateManager:new(core)
  self.core = core
  self.stack = {}
end


function StateManager:push(state)
  self.stack[#self.stack+1] = state
  self.stack[#self.stack]:enter()
end


function StateManager:top()
  return self.stack[#self.stack]
end


function StateManager:pop()
  self.stack[#self.stack]:leave()
  self.stack[#self.stack] = nil

end


function StateManager:replace(state)
  self:pop()
  self:push(state)
end


function StateManager:update(dt)
  if #self.stack == 0 then
    return false
  end

  self.stack[#self.stack]:update(dt)

  return true
end


function StateManager:draw()
  if #self.stack > 0 then
    self.stack[#self.stack]:draw()
  end
end


function StateManager:mousereleased(x,y,button)
  local s = self.stack[#self.stack]
  local mr = s.mousereleased 
  if mr then
    mr(s,x,y,button)
  end
end


function StateManager:mousepressed(x,y,button)
  local s = self.stack[#self.stack]
  local mp = s.mousepressed 
  if mp then
    mp(s, x,y,button)
  end
end


function StateManager:mousemoved(x,y)
  local s = self.stack[#self.stack]
  local mm = s.mousemoved
  if mm then
    mm(s,x,y)
  end
end

return StateManager
