--
-- lust
--
-- Copyright (c) 2014, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

--- Manager for subsystems. Allows updating them all with one call.
--  Next release will also allow disabling single subsystems without
--  removing them from the chain.
local SubsystemManager = Object:extend()

--- Create new SubsystemManager. Call SubsystemManager(...) to call constructor
--  and obtain the new instance.
function SubsystemManager:new()
  self.active_subsystems = {}
  self.passive_subsystems = {}
end

--- Create a new Subsystem within this manager. Does not yet do very much
--  besides creating the instance and adding it to the manager, but will
--  be extended in the near future.
--  @param subsystem_type Callable that produces the new subsystem. Will
--                        usually be a sub-class of Subsystem
--  @param ...            Parameters that are passed verbatim to the
--                        subsystem constructor
function SubsystemManager:create_subsystem(subsystem_type, ...)
  local new_subsys = subsystem_type(...)
  self.active_subsystems[#self.active_subsystems+1] = new_subsys;
  return new_subsys
end

--- Update all subsystems
--  @param dt Delta time since last update in seconds
function SubsystemManager:update(dt)
  for k,s in pairs(self.active_subsystems) do
    s:update(dt)
  end
end

return SubsystemManager
