--
-- lust
--
-- Copyright (c) 2014, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

Text = Object:extend()

function Text:new(font)
  self.blobs = {}
  self.tween = flux.group()
  self.font = font
  self.background = {0,0,0,128}
  self.defaultColor = {255,255,255,255}
end


function Text:update(dt)
  for t in pairs(self.blobs) do
    if t.vx then
      t.x = t.x + t.vx * dt
    end

    if t.vy then
      t.y = t.y + t.vy *dt
    end

    if t.timeout > 0 then
      t.timeout = t.timeout - dt
      if t.timeout <= 0 then
        self.tween:to(t, t.fadespeed or self.fadespeed or 2, {alpha = 0.0})
          :oncomplete(function() t.dead = true ; self.blobs[t] = nil end)
      end
    end
  end

  self.tween:update(dt)
end


function Text:draw()
  love.graphics.setFont(self.font)
  local h = self.font:getHeight()
  for t in pairs(self.blobs) do
    local w = self.font:getWidth(t.text)
    local br,bg,bb,ba = unpack(t.bgcolor or self.background)
    local r,g,b,a = unpack(t.color)
    love.graphics.setColor(br,bg,bb,ba*t.alpha)
    local x
    if self.restrict then
      x = math.max(self.restrict[1],math.min(self.restrict[2]-w-2, t.x - w / 2 -2 ))
    else
      x = 0, t.x - w / 2 -2
    end
    local y = math.max(math.floor(t.y -h / 2), 0)

    love.graphics.rectangle('fill', math.floor(x), math.floor(y), w+4, h+4)
    love.graphics.setColor(r,g,b,a*t.alpha)
    love.graphics.print(t.text, math.floor(x + 2), math.floor(y +2) )
  end
end


function Text:remove(blob)
  self.blobs[blob] = nil
end


local BlobMT = { }
BlobMT.__index = BlobMT
BlobMT.kill = function(self) self.timeout = 0 end


function Text:add(text, x, y, timeout, vx, vy, color, oldblob)
  local blob = {text = text, x = x, y = y, timeout = timeout, alpha = 1.0, vx = vx, vy = vy, color = color or self.defaultColor}
  if oldblob then
    self.blobs[oldblob] = nil
  end
  setmetatable(blob, BlobMt)
  self.blobs[blob] = blob
  return blob
end


return Text
