--
-- lust
--
-- Copyright (c) 2014, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local Menu = require('lust.menu')

local TextMenuScreen = Menu.Screen:extend()
local BoundInput = require('lust.boundinput')

function TextMenuScreen:new(title, text)
  self.title = title
  self.text = text
  self.input = BoundInput()
  self.input:bind_key('down', function() self.scrollSpeed = self.scrollSpeed + 1 end,
                              function() self.scrollSpeed = self.scrollSpeed - 1 end)
  self.input:bind_key('up', function() self.scrollSpeed = self.scrollSpeed - 1 end,
                            function() self.scrollSpeed = self.scrollSpeed + 1 end)
  self.input:bind_key('pagedown', function() self.offset = self.offset + 10 end)
  self.input:bind_key('pageup', function() self.offset = self.offset - 10 end)
  self.input:bind_key('escape', function() self.menu:leaveScreen() end)
end


function TextMenuScreen:enter(menu)
  TextMenuScreen.super.enter(self, menu)
  self.offset = 0
  menu.core.input:push(self.input)
  self.menu = menu
  self.scrollSpeed = 0

end


function TextMenuScreen:leave()
  self.menu.core.input:pop()
end


function TextMenuScreen:update(dt)
  self.offset = self.offset + dt * 10 * self.scrollSpeed
  self.offset = math.max(0, self.offset)
  self.offset = math.min(#self.text, self.offset)
end

--function TextMenuScreen:input


function TextMenuScreen:draw(style)
  style:drawTitle(self.title)

  local offset = math.floor(self.offset)
  local rowsPerScreen = style:rowsPerScreen()
  offset = math.min(offset, #self.text - rowsPerScreen)

  for i = math.max(1, offset + 1), math.min(offset + 1 + rowsPerScreen, #self.text) do
    style:drawCaption(self.text[i], false)
  end

  style.y = style.style.height - style.style.entryFont:getHeight() 
  style:drawCaption("[escape]:return, [up]/[down]:scroll", true)
end

return TextMenuScreen
