--
-- lust
--
-- Copyright (c) 2014, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local function addCollisionShape(collider, shape)
  local shape = collider:addPolygon(unpack(shape))
  collider:addToGroup("level", shape)
  shape.islevel = true
  collider:setPassive(shape)
end


local function addLeftCollisionLine(collider, x1, y1, y2, tileWidth, tileHeight)
  local shape = {x1,y1+1,
                 x1,y2-1,
                 x1+tileWidth/2,y2-tileHeight/2,
                 x1+tileWidth/2,y1+tileHeight/2}
  if y2 == y1 + tileHeight then
    shape = {x1,y1+1,
             x1,y2-1,
             x1+tileWidth/2,y1+tileHeight/2}
  end
  addCollisionShape(collider, shape)
end


local function addRightCollisionLine(collider, x1,y1,y2, tileWidth, tileHeight)
  local shape = {x1+tileWidth,y1+1,
                 x1+tileWidth,y2-1,
                 x1+tileWidth/2,y2-tileHeight/2,
                 x1+tileWidth/2,y1+tileHeight/2}
  if y2 == y1 + tileHeight then
    shape = {x1+tileWidth,y1+1,
             x1+tileWidth,y2-1,
             x1+tileWidth/2,y1+tileHeight/2}
  end
  addCollisionShape(collider, shape)
end


local function addTopCollisionLine(collider, x1,y1,x2, tileWidth, tileHeight)
  local shape = {x1+1,y1,
                 x1+tileWidth/2,y1+tileHeight/2,
                 x2-tileWidth/2,y1+tileHeight/2,
                 x2-1, y1}
  if x2 == x1 + tileWidth then
    shape = {x1+1,y1,
             x1+tileWidth/2,y1+tileHeight/2,
             x2-1, y1}
                            
  end
  addCollisionShape(collider, shape)
end


local function addBottomCollisionLine(collider, x1,y1,x2, tileWidth, tileHeight)
  local shape = {x1+1,     y1 + tileHeight,
                 x2-1,     y1 + tileHeight,
                 x2 - tileWidth/2, y1 + tileHeight/2,
                 x1 + tileWidth/2, y1 + tileHeight/2}

  if x2 == x1 + tileWidth then
    shape = {x1+1, y1 + tileHeight,
                   x2-1, y1 + tileHeight/2,
                   x1 +tileWidth/2, y1+tileHeight/2}
  end
  addCollisionShape(collider, shape)
end




local function buildTilemapCollisionsMesh(collider, width, height, tileWidth, tileHeight, getTile, isPenetrable)

  -- Horizontal line segments
  for r = 1, height do
    local firstcolabove = 0
    local firstcolbelow = 0

    for c = 1, width do
      if not isPenetrable(getTile(c,r)) and isPenetrable(getTile(c, r - 1)) then
        if firstcolabove == 0 then
          firstcolabove = c
        end
      else
        if firstcolabove ~= 0 then
          addTopCollisionLine(collider, (firstcolabove-1) * tileWidth, (r-1) * tileHeight, (c-1) * tileWidth, tileWidth, tileHeight)
          firstcolabove = 0
        end
      end

      if not isPenetrable(getTile(c,r)) and isPenetrable(getTile(c, r + 1)) then
        if firstcolbelow == 0 then
          firstcolbelow = c
        end
      else
        if firstcolbelow ~= 0 then
          addBottomCollisionLine(collider, (firstcolbelow-1) * tileWidth, (r-1) * tileHeight, (c-1) * tileWidth, tileWidth, tileHeight)
          firstcolbelow = 0
        end
      end
    end
  end

  -- Vertical line segments
  for c = 1, width do
    local firstrowleft = 0
    local firstrowright = 0

    for r = 1, height do
      if not isPenetrable(getTile(c,r)) and isPenetrable(getTile(c - 1, r)) then
        if firstrowleft == 0 then
          firstrowleft = r
        end
      else
        if firstrowleft ~= 0 then
          addLeftCollisionLine(collider, (c-1) * tileWidth, (firstrowleft-1) * tileHeight, (r-1) * tileHeight, tileWidth, tileHeight)
          firstrowleft = 0
        end
      end

      if not isPenetrable(getTile(c,r)) and isPenetrable(getTile(c + 1, r)) then
        if firstrowright == 0 then
          firstrowright = r
        end
      else
        if firstrowright ~= 0 then
          addRightCollisionLine(collider, (c-1) * tileWidth, (firstrowright-1) * tileHeight, (r-1) * tileHeight, tileWidth, tileHeight)
          firstrowright = 0
        end
      end
    end
  end
end

return buildTilemapCollisionsMesh
