require "lust"
local CRTRenderer = require "crt_renderer"
local Game = require "game"

function love.load()
  G = {}
  G.crt = CRTRenderer()
  local w,h = love.graphics.getDimensions()
  G.background = love.graphics.newCanvas(w,h)
  love.graphics.setCanvas(G.background)
  love.graphics.setLineWidth(2)

  s = 0
  e = 1
  love.graphics.setBlendMode('alpha')
  love.graphics.setColor(255,255,255,64.0 / 255.0)
  for i = 1, 16 do
    love.graphics.line(w * s,       h * i / 16, w * e,      h * i / 16)
    love.graphics.line(w * i / 16, h * s,       w * i / 16, h * e)
  end
  love.graphics.setColor(255,255,255,255)

  love.graphics.setCanvas()
  G.a = 0
  G.game = Game()

  G.titlefont = love.graphics.newFont("Vectorb.ttf", 50)
  G.font = love.graphics.newFont("Vectorb.ttf", 20)
  love.graphics.setFont(G.font)

  G.sounds = {
    jump1 = love.audio.newSource("jump1.ogg", "static"),
    jump2 = love.audio.newSource("jump2.ogg", "static"),
    blip = love.audio.newSource("blip.ogg", "static"),
    land = love.audio.newSource("land.ogg", "static"),
    die = love.audio.newSource("die.ogg", "static"),
    bonus = love.audio.newSource("bonus.ogg", "static"),
    getbonus = love.audio.newSource("getbonus.ogg", "static"),
    getbonus2 = love.audio.newSource("getbonus2.ogg", "static"),
    save = love.audio.newSource("save.ogg", "static"),
    levelup = love.audio.newSource("levelup.ogg", "static")
  }
end


function love.update(dt)

--[[
  local points = {}
  for i = 0, 4 do
    table.insert(points, math.sin(G.a + i / 100) * 128 + 256)
    table.insert(points, math.cos(3*(G.a + i / 100)) * 128 + 256)
  end

  G.crt:line(points)

  G.a = G.a + 0.045
--]]

  G.game:update(dt)

  G.crt:update(dt)
end


function love.keypressed(key)
  if key == 'r' then
    G.game = Game()
  else
    G.game:keypressed(key)
  end
end

function love.keyreleased(key)
  G.game:keyreleased(key)
end


function love.draw()
  ---[[
  love.graphics.setCanvas()
  --love.graphics.clear()
  love.graphics.setShader()
  ---[[
  love.graphics.setBlendMode('replace')
  love.graphics.setColor(255,255,255,255)
  love.graphics.draw(G.background)
  --]]
  --G.crt:print(G.font, 0.3, love.timer.getFPS(), 10, 10)
  G.crt:execute()

end
